using Gtk;


namespace Eidete.Widgets {

	private bool contractor;

	public class EndDialog : Window {
		public EndDialog (EideteApp app){
			this.set_default_size (400, 200);

			this.set_application (app);

			if (app.selectionarea != null)
				app.selectionarea.destroy ();
			if (app.keyview != null)
				app.keyview.destroy ();
			this.window_position = WindowPosition.CENTER;
			this.icon_name = "eidete";

			var grid = new Grid ();
			grid.margin = 12;

			var title = new Label ("<span size='30000'>"+_("Recording complete")+"</span>");
			title.use_markup = true;
			title.halign = Align.START;

			var export = new Button.with_label (_("Save"));
			export.image = new Image.from_stock (Stock.SAVE, IconSize.BUTTON);
			export.can_default = true;
			this.set_default (export);

			var cancel = new Button.with_label (_("Don't save"));
			cancel.image = new Image.from_stock (Stock.DELETE, IconSize.BUTTON);
			cancel.halign = Align.END;

			var bbox = new ButtonBox (Orientation.HORIZONTAL);
			bbox.pack_end (cancel, false, true, 0);
			bbox.pack_end (export, false, true, 0);
			bbox.layout_style = ButtonBoxStyle.END;

			//contractor
			var list = new ListStore (2, typeof (Gdk.Pixbuf), typeof (string));
			var t = new TreeView.with_model (list);
			t.headers_visible = false;
			t.hexpand = true;

			var cell1 = new CellRendererPixbuf ();
			cell1.set_padding (5, 15);
			t.insert_column_with_attributes (-1, "", cell1, "pixbuf", 0);
			var cell2 = new CellRendererText ();
			cell2.set_padding (2, 15);
			t.insert_column_with_attributes (-1, "", cell2, "markup", 1);


			var c = Granite.Services.Contractor.get_contract (app.settings.destination, "video");
			if (c == null || c.length <= 1){
				warning ("You should install and/or run contractor");
				contractor = false;
				var info = new InfoBar ();
				info.message_type = MessageType.WARNING;
				info.pack_start (new Label (_("Could not contact Contractor. Click the Save button to save the file.")));
				grid.attach (info, 0, 3, 2, 1);
				export.label = _("Save");
			}else{
				contractor = true;
				for (var i=0;i<c.length; i++){
					TreeIter it;
					list.append (out it);
					Gdk.Pixbuf icon = null;
					try{
						icon = IconTheme.get_default ().load_icon (c[i].lookup ("IconName"), 32, 0);
					}catch (Error e){warning (e.message);}
					list.set (it, 0, icon, 1,
					    "<b>"+c[i].lookup ("Name")+"</b>\n"+c[i].lookup ("Description"));
				}
				t.set_cursor (new TreePath.from_string ("0"), null, false);
			}

			grid.attach (title, 0, 0, 2, 1);
			grid.attach (new Label (""), 0, 2, 1, 1);
			grid.attach (t, 0, 4, 1, 1);
			grid.attach (new Label (""), 0, 5, 1, 1);
			grid.attach (bbox, 0, 6, 1, 1);

			if (contractor){
				export.clicked.connect ( () => {
					TreePath path;
					t.get_cursor (out path, null);
					try{
						Process.spawn_command_line_async (c[int.parse (path.to_string ())].lookup ("Exec"));
					}catch (Error e){
						print(e.message);
					}
					this.destroy ();
				});
			} else {
				export.clicked.connect ( () => {
					var source = File.new_for_path (app.settings.destination);
					var file = new FileChooserDialog (_("Save"), null, FileChooserAction.SAVE, Stock.OK, ResponseType.OK);
					file.set_current_name (source.get_basename ());
					file.do_overwrite_confirmation = true;
					var res = file.run ();
					if (res == ResponseType.OK){
						var destination = File.new_for_path (file.get_filename ());
						try{
							source.copy (destination, FileCopyFlags.OVERWRITE);
						}catch (GLib.Error e){stderr.printf ("Error: %s\n", e.message);}

						file.destroy ();
						this.destroy ();
					}else{
						file.destroy ();
					}
				});
			}
			cancel.clicked.connect ( () => {this.destroy  ();});

			this.add (grid);
		}
	}
}


