using Gtk;

public class Uploader : Window{
	
	class LLabel : Label{
		public LLabel (string label){
			this.set_halign (Align.START);
			this.label = label;
		}
		public LLabel.indent (string label){
			this (label);
			this.margin_left = 10;
		}
		public LLabel.markup (string label){
			this (label);
			this.use_markup = true;
		}
		public LLabel.right (string label){
			this.set_halign (Align.END);
			this.label = label;
		}
		public LLabel.right_with_markup (string label){
			this.set_halign (Align.END);
			this.use_markup = true;
			this.label = label;
		}
	}
	
	
	public Uploader (string file){
		this.title = "Upload to videobin.org";
		this.set_default_size (300, 1);
		this.window_position = WindowPosition.CENTER;
		
		var grid = new Grid ();
		grid.margin = 12;
		grid.column_spacing = 12;
		grid.row_spacing = 5;
		
		var title = new Entry ();
		var description = new Entry ();
		var email = new Entry ();
		var img = new Image.from_icon_name ("videobin", IconSize.DIALOG);
		
		var upload = new Button.with_label (_("Upload"));
		var cancel = new Button.from_stock (Stock.CANCEL);
		
		var bbox = new Box (Orientation.HORIZONTAL, 0);
		bbox.pack_end (upload, false, true, 0);
		bbox.pack_end (cancel, false, true, 0);
		
		email.set_tooltip_text ("Your email address for relocating your videos");
		upload.image = new Image.from_icon_name ("mail-reply-sender", IconSize.BUTTON);
		upload.can_default = true;
		this.set_default (upload);
		
		grid.attach (img, 0, 0, 1, 2);
		grid.attach (new LLabel ("Title"), 1, 0, 1, 1);
		grid.attach (title, 2, 0, 1, 1);
		grid.attach (new LLabel ("Email"), 1, 1, 1, 1);
		grid.attach (email, 2, 1, 1, 1);
		grid.attach (new LLabel ("Description"), 1, 2, 1, 1);
		grid.attach (description, 2, 2, 1, 1);
		grid.attach (bbox, 0, 3, 3, 1);
		
		this.add (grid);
		this.destroy.connect (Gtk.main_quit);
		
		cancel.clicked.connect (Gtk.main_quit);
		
		upload.clicked.connect ( () => {
			string url;
			string command = "curl -F\"api=1\" -F\"videoFile=@"+file+"\" ";
			if (email.text != "")
				command += "-F\"email="+email.text+"\" ";
			if (title.text != "")
				command += "-F\"title="+title.text+"\" ";
			if (description.text != "")
				command += "-F\"description="+description.text+"\" ";
			command += "http://videobin.org/add";
			
			try{
				Process.spawn_command_line_sync (command, out url);
			}catch (Error e){error (e.message);}
			try{
				Process.spawn_command_line_async ("sensible-browser "+url);
			}catch (Error e){error (e.message);}
			Gtk.main_quit ();
		});
	}
	
}

public static void main (string [] args){
	Gtk.init (ref args);
	var file = File.new_for_uri (args[1]);
	var u = new Uploader (file.get_path ());
	u.show_all ();
	Gtk.main ();
}

